package org.jtheque.books;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.books.services.able.IAuthorsService;
import org.jtheque.books.services.able.IBooksService;
import org.jtheque.books.services.able.IEditorsService;
import org.jtheque.primary.services.able.IKindsService;
import org.jtheque.primary.services.able.ILendingsService;
import org.jtheque.primary.services.able.ISagasService;
import org.jtheque.primary.services.able.ITypesService;
import org.jtheque.books.services.impl.utils.config.ConfigManager;
import org.jtheque.books.services.impl.utils.config.Configuration;
import org.jtheque.books.persistence.BooksSchema;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.language.ILanguageManager;
import org.jtheque.core.managers.log.ILoggingManager;
import org.jtheque.core.managers.schema.Schema;
import org.jtheque.core.managers.schema.ISchemaManager;
import org.jtheque.core.managers.error.InternationalizedError;
import org.jtheque.core.managers.error.IErrorManager;
import org.jtheque.core.managers.feature.Feature;
import org.jtheque.core.managers.feature.IFeatureManager;
import org.jtheque.core.managers.feature.IFeatureManager.CoreFeature;
import org.jtheque.core.managers.feature.Feature.FeatureType;
import org.jtheque.core.managers.module.annotations.ModuleDefinition;
import org.jtheque.core.managers.module.annotations.PrimaryModule;
import org.jtheque.core.managers.module.annotations.Module;
import org.jtheque.core.managers.module.annotations.PrePlug;
import org.jtheque.core.managers.module.annotations.Plug;
import org.jtheque.core.managers.module.annotations.UnPlug;
import org.jtheque.core.managers.state.StateException;
import org.jtheque.core.managers.state.IStateManager;
import org.jtheque.core.managers.view.IViewManager;
import org.jtheque.core.managers.view.able.components.TabComponent;
import org.jtheque.primary.view.impl.sort.Sorter;
import org.jtheque.primary.view.impl.sort.SorterFactory;
import org.jtheque.primary.view.impl.choice.ChoiceAction;
import org.jtheque.primary.view.impl.choice.ChoiceActionFactory;
import org.jtheque.primary.PrimaryUtils;
import org.jtheque.primary.utils.DataTypeManager;

/**
 * A JTheque Module for managing books.
 *
 * @author Baptiste Wicht
 */
@Module(name = "JTheque Books Module", author = "Baptiste Wicht", description = "Module de gestion de livres", 
    version = "1.3", coreVersion = "2.0.1", jarFile = "jtheque-books-module-1.3.jar")
@PrimaryModule(icon = "org/jtheque/books/ressources/images/BooksModule.png")
@ModuleDefinition(updateURL = "http://jtheque.developpez.com/public/versions/BooksModule.versions")
public final class BooksModule {
    public static final String IMAGE_BASE_NAME = "org/jtheque/books/ressources/images";
    private static final String BASENAME = "org.jtheque.books.ressources.i18n.books";
    
    private ConfigManager config;
    
    //Features
    private Feature refreshFeature;
    private Feature othersFeature;

    private Schema schema;
    
    private TabComponent[] tabComponents;
    private Sorter[] sorters;
    private ChoiceAction[] choiceActions;

    @PrePlug
    public void prePlug() {
        Managers.getManager(ILanguageManager.class).addBasename(BASENAME);

        schema = new BooksSchema();

        Managers.getManager(ISchemaManager.class).registerSchema(schema);

        PrimaryUtils.setPrimaryImpl("Books");
        PrimaryUtils.prePlug();
    }

    @Plug
    public void plug() {
        config = Managers.getManager(IStateManager.class).getState(ConfigManager.class);

        if (config == null) {
            try {
                config = Managers.getManager(IStateManager.class).createState(ConfigManager.class);
            } catch (StateException e) {
                Managers.getManager(ILoggingManager.class).getLogger(getClass()).exception(e);
                config = new ConfigManager();
                Managers.getManager(IErrorManager.class).addError(new InternationalizedError("error.loading.configuration"));
            }

            config.setDefaults();
        }

        PrimaryUtils.plug();

        DataTypeManager.bindDataTypeToKey(IAuthorsService.DATA_TYPE, "data.titles.author");
        DataTypeManager.bindDataTypeToKey(IBooksService.DATA_TYPE, "data.titles.book");
        DataTypeManager.bindDataTypeToKey(IEditorsService.DATA_TYPE, "data.titles.editor");
        DataTypeManager.bindDataTypeToKey(IKindsService.DATA_TYPE, "data.titles.kind");
        DataTypeManager.bindDataTypeToKey(ILendingsService.DATA_TYPE, "data.titles.lending");
        DataTypeManager.bindDataTypeToKey(ISagasService.DATA_TYPE, "data.titles.saga");
        DataTypeManager.bindDataTypeToKey(ITypesService.DATA_TYPE, "data.titles.type");
        
        for(Sorter sorter : sorters){
            SorterFactory.getInstance().addSorter(sorter);
        }
        
        /* Menus */

        //File

        IFeatureManager feature = Managers.getManager(IFeatureManager.class);
        
        refreshFeature = feature.addSubFeature(feature.getFeature(CoreFeature.FILE), "refreshAction", FeatureType.ACTION, 100);
        
        //Others

        othersFeature = feature.createFeature(500, FeatureType.PACK, "actions.others");

        Feature newFeature = feature.createFeature(1, FeatureType.ACTIONS, "actions.others.new");
        
        feature.addSubFeature(newFeature, "newKindAction", FeatureType.ACTION, 1);
        feature.addSubFeature(newFeature, "newTypeAction", FeatureType.ACTION, 2);
        feature.addSubFeature(newFeature, "newLanguageAction", FeatureType.ACTION, 3);
        feature.addSubFeature(newFeature, "newCountryAction", FeatureType.ACTION, 4);
        feature.addSubFeature(newFeature, "newEditorAction", FeatureType.ACTION, 5);

        othersFeature.addSubFeature(newFeature);

        Feature deleteFeature = feature.createFeature(2, FeatureType.ACTIONS, "actions.others.delete");
        
        feature.addSubFeature(deleteFeature, "deleteKindAction", FeatureType.ACTION, 1);
        feature.addSubFeature(deleteFeature, "deleteTypeAction", FeatureType.ACTION, 2);
        feature.addSubFeature(deleteFeature, "deleteLanguageAction", FeatureType.ACTION, 3);
        feature.addSubFeature(deleteFeature, "deleteCountryAction", FeatureType.ACTION, 4);
        feature.addSubFeature(deleteFeature, "deleteEditorAction", FeatureType.ACTION, 5);

        othersFeature.addSubFeature(deleteFeature);

        Feature editFeature = feature.createFeature(3, FeatureType.ACTIONS, "actions.others.modify");
        
        feature.addSubFeature(editFeature, "editKindAction", FeatureType.ACTION, 1);
        feature.addSubFeature(editFeature, "editTypeAction", FeatureType.ACTION, 2);
        feature.addSubFeature(editFeature, "editLanguageAction", FeatureType.ACTION, 3);
        feature.addSubFeature(editFeature, "editCountryAction", FeatureType.ACTION, 4);
        feature.addSubFeature(editFeature, "editEditorAction", FeatureType.ACTION, 5);

        othersFeature.addSubFeature(editFeature);

        Managers.getManager(IFeatureManager.class).addFeature(othersFeature);
        
        for (ChoiceAction action : choiceActions) {
            ChoiceActionFactory.addChoiceAction(action);
        }

        for(TabComponent component : tabComponents){
            Managers.getManager(IViewManager.class).addTabComponent(component);
        }
    }

    @UnPlug
    public void unplug() {
        for (ChoiceAction action : choiceActions) {
            ChoiceActionFactory.removeChoiceAction(action);
        }
        
        Managers.getManager(IFeatureManager.class).removeFeature(othersFeature);
        Managers.getManager(IFeatureManager.class).getFeature(CoreFeature.FILE).removeSubFeature(refreshFeature);

        DataTypeManager.unbindDataType(IAuthorsService.DATA_TYPE);
        DataTypeManager.unbindDataType(IBooksService.DATA_TYPE);
        DataTypeManager.unbindDataType(IEditorsService.DATA_TYPE);
        DataTypeManager.unbindDataType(IKindsService.DATA_TYPE);
        DataTypeManager.unbindDataType(ILendingsService.DATA_TYPE);
        DataTypeManager.unbindDataType(ISagasService.DATA_TYPE);
        DataTypeManager.unbindDataType(ITypesService.DATA_TYPE);

        for(Sorter sorter : sorters){
            SorterFactory.getInstance().removeSorter(sorter);
        }
        
        for(TabComponent tabComponent : tabComponents){
            Managers.getManager(IViewManager.class).removeTabComponent(tabComponent);
        }
        
        PrimaryUtils.unplug();

        Managers.getManager(ISchemaManager.class).unregisterSchema(schema);

        Managers.getManager(ILanguageManager.class).removeBasename(BASENAME);
    }

    /**
     * Return the configuration of the module.
     *
     * @return The configuration of the module.
     */
    public Configuration getConfiguration() {
        return config;
    }

    public void setTabComponents(TabComponent[] tabComponents) {
        this.tabComponents = tabComponents;
    }

    public void setSorters(Sorter[] sorters) {
        this.sorters = sorters;
    }

    public void setChoiceActions(ChoiceAction[] choiceActions) {
        this.choiceActions = choiceActions;
    }
}