package org.jtheque.books.persistence;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.schema.Schema;
import org.jtheque.core.managers.schema.Insert;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.beans.IBeansManager;
import org.jtheque.utils.bean.Version;
import org.jtheque.books.persistence.dao.able.IDaoEditors;
import org.jtheque.books.persistence.dao.able.IDaoBooks;
import org.jtheque.primary.dao.able.*;
import org.jtheque.books.persistence.dao.able.IDaoAuthors;

import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;

import javax.annotation.Resource;

/**
 * The database schema for the Movies Module.
 *
 * @author Baptiste Wicht
 */
public final class BooksSchema implements Schema {
    @Resource
    private SimpleJdbcTemplate jdbcTemplate;

    /**
     * Construct a new MoviesSchema.
     */
    public BooksSchema(){
        super();

        Managers.getManager(IBeansManager.class).inject(this);
    }

    @Override
    public Version getVersion() {
        return new Version("1.1");
    }

    @Override
    public String getName() {
        return "Books-Schema";
    }

    @Override
    public void install() {
        createDataTable();
        createReferentialIntegrityConstraints();
    }

    @Override
    public void update(Version from) {
        if("1.0".equals(from.getVersion())){
            createReferentialIntegrityConstraints();            
        }
    }

    private void createDataTable() {
        jdbcTemplate.update("CREATE TABLE " + IDaoEditors.TABLE + " (ID INT IDENTITY PRIMARY KEY, NAME VARCHAR(150) NOT NULL UNIQUE)");
        jdbcTemplate.update("CREATE TABLE " + IDaoAuthors.TABLE + " (ID INT IDENTITY PRIMARY KEY, NAME VARCHAR(100) NOT NULL, FIRSTNAME VARCHAR(100) NOT NULL, NOTE INT, THE_COUNTRY_FK INT NULL, CONSTRAINT UNIQUE_AUTHOR UNIQUE(NAME, FIRSTNAME))");
        jdbcTemplate.update("CREATE TABLE " + IDaoBooks.TABLE + " (ID INT IDENTITY PRIMARY KEY, TITLE VARCHAR(150) NOT NULL UNIQUE, YEAR INT, ISBN10 VARCHAR(20), ISBN13 VARCHAR(13), NOTE INT, PAGES_NUMBER INT, RESUME VARCHAR(2000), THE_EDITOR_FK INT, THE_KIND_FK INT, THE_LANGUAGE_FK INT, THE_LENDING_FK INT, THE_SAGA_FK INT, THE_TYPE_FK INT)");
        jdbcTemplate.update("CREATE TABLE " + IDaoBooks.BOOKS_AUTHOR_TABLE + " (THE_BOOK_FK INT NOT NULL, THE_AUTHOR_FK INT NOT NULL)");

        jdbcTemplate.update("CREATE INDEX BOOK_EDITOR_IDX ON " + IDaoEditors.TABLE + "(ID)");
        jdbcTemplate.update("CREATE INDEX BOOK_AUTHORS_IDX ON " + IDaoAuthors.TABLE + "(ID)");
        jdbcTemplate.update("CREATE INDEX BOOKS_IDX ON " + IDaoBooks.TABLE + "(ID)");
    }

    private void createReferentialIntegrityConstraints() {
        jdbcTemplate.update("ALTER TABLE " + IDaoBooks.BOOKS_AUTHOR_TABLE + " ADD FOREIGN KEY (THE_BOOK_FK) REFERENCES  " + IDaoBooks.TABLE + "  (ID) ON UPDATE SET NULL");
        jdbcTemplate.update("ALTER TABLE " + IDaoBooks.BOOKS_AUTHOR_TABLE + " ADD FOREIGN KEY (THE_AUTHOR_FK) REFERENCES  " + IDaoAuthors.TABLE + "  (ID) ON UPDATE SET NULL");
        
        jdbcTemplate.update("ALTER TABLE " + IDaoAuthors.TABLE + " ADD FOREIGN KEY (THE_COUNTRY_FK) REFERENCES  " + IDaoCountries.TABLE + "  (ID) ON UPDATE SET NULL");
        
        jdbcTemplate.update("ALTER TABLE " + IDaoBooks.TABLE + " ADD FOREIGN KEY (THE_KIND_FK) REFERENCES  " + IDaoKinds.TABLE + "  (ID) ON UPDATE SET NULL");
        jdbcTemplate.update("ALTER TABLE " + IDaoBooks.TABLE + " ADD FOREIGN KEY (THE_LANGUAGE_FK) REFERENCES  " + IDaoLanguages.TABLE + "  (ID) ON UPDATE SET NULL");
        jdbcTemplate.update("ALTER TABLE " + IDaoBooks.TABLE + " ADD FOREIGN KEY (THE_LENDING_FK) REFERENCES  " + IDaoLendings.TABLE + "  (ID) ON UPDATE SET NULL");
        jdbcTemplate.update("ALTER TABLE " + IDaoBooks.TABLE + " ADD FOREIGN KEY (THE_SAGA_FK) REFERENCES  " + IDaoSagas.TABLE + "  (ID) ON UPDATE SET NULL");
        jdbcTemplate.update("ALTER TABLE " + IDaoBooks.TABLE + " ADD FOREIGN KEY (THE_TYPE_FK) REFERENCES  " + IDaoTypes.TABLE + "  (ID) ON UPDATE SET NULL");
        jdbcTemplate.update("ALTER TABLE " + IDaoBooks.TABLE + " ADD FOREIGN KEY (THE_EDITOR_FK) REFERENCES  " + IDaoEditors.TABLE + "  (ID) ON UPDATE SET NULL");
    }
    
    @Override
    public void importDataFromHSQL(Iterable<Insert> inserts) {
        for(Insert insert : inserts){
            if("OD_SAGA".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoSagas.TABLE + " (ID, NAME, IMPL) VALUES (?,?,?)", insert.getInt(0), insert.getString(2), "Books");
            } else if("OD_EDITOR".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoEditors.TABLE + " (ID, NAME) VALUES (?,?,?)", insert.getInt(0), insert.getString(2));
            } else if("OD_KIND_BOOK".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoKinds.TABLE + " (ID, NAME, IMPL) VALUES (?,?)", insert.getInt(0), insert.getString(2), "Books");
            } else if("OD_TYPE_BOOK".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoTypes.TABLE + " (ID, NAME, IMPL) VALUES (?,?,?)", insert.getInt(0), insert.getString(2), "Books");
            } else if("BOOK_AUTHOR".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoBooks.BOOKS_AUTHOR_TABLE + " (THE_BOOK_FK, THE_AUTHOR_FK) VALUES (?,?)", insert.getInt(0), insert.getInt(1));
            } else if("OD_LENDING_BOOK".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoLendings.TABLE + " (ID, DATE, THE_BORROWER_FK, IMPL) VALUES (?,?,?,?)", insert.getInt(0), insert.getInt(2), insert.getInt(3), "Books");
            } else if("OD_AUTHOR".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoAuthors.TABLE + " (ID, NAME, FIRSTNAME, NOTE, THE_COUNTRY_FK) VALUES (?,?,?,?,?)",
                        insert.getInt(0), insert.getString(3), insert.getString(2), insert.getInt(4), insert.getInt(5));
            } else if("OD_BOOK".equals(insert.getTable())){
                jdbcTemplate.update("INSERT INTO " + IDaoBooks.TABLE + " (TITLE, YEAR, ISBN10, ISBN13, NOTE, PAGES_NUMBER, RESUME, THE_EDITOR_FK, THE_KIND_FK, THE_LANGUAGE_FK, THE_LENDING_FK, THE_SAGA_FK, THE_TYPE_FK) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?)",
                        insert.getString(7), insert.getInt(8), insert.getString(2), insert.getString(3), insert.getInt(4), insert.getInt(5), insert.getString(6), insert.getInt(9),
                        insert.getInt(10), insert.getInt(11), insert.getInt(12), insert.getInt(13), insert.getInt(14));
            }
        }
    }
}