package org.jtheque.books.persistence.dao.impl;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.books.persistence.dao.able.IDaoEditors;
import org.jtheque.books.persistence.od.EditorImpl;
import org.jtheque.books.persistence.od.abstraction.Editor;
import org.jtheque.core.managers.persistence.Entity;
import org.jtheque.core.managers.persistence.GenericDao;
import org.jtheque.core.managers.persistence.Query;
import org.jtheque.core.managers.persistence.QueryMapper;
import org.jtheque.core.managers.persistence.context.IDaoPersistenceContext;

import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;

import javax.annotation.Resource;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

/**
 * Cette classe permet de grer la persistance des genres.
 *
 * @author Baptiste Wicht
 */
public final class DaoEditors extends GenericDao<EditorImpl> implements IDaoEditors {
    private final ParameterizedRowMapper<EditorImpl> rowMapper = new EditorRowMapper();
    private final QueryMapper queryMapper = new EditorQueryMapper();

    @Resource
    private IDaoPersistenceContext persistenceContext;

    @Resource
    private SimpleJdbcTemplate jdbcTemplate;

    /**
     * Construct a new DaoEditors.
     */
    public DaoEditors() {
        super(TABLE);
    }

    @Override
    public boolean exists(String name) {
        return getEditor(name) != null;
    }

    @Override
    public List<EditorImpl> getEditors() {
        return getAll();
    }

    @Override
    public EditorImpl getEditor(int id) {
        return get(id);
    }

    @Override
    public EditorImpl getEditor(String name) {
        List<EditorImpl> editors = jdbcTemplate.query("SELECT * FROM " + TABLE + " WHERE NAME = ?", rowMapper, name);

        if(editors.isEmpty()){
            return null;
        }
        
        EditorImpl editor = editors.get(0);
        
        if (isNotInCache(editor.getId())) {
            getCache().put(editor.getId(), editor);
        }

        return getCache().get(editor.getId());
    }

    @Override
    protected ParameterizedRowMapper<EditorImpl> getRowMapper() {
        return rowMapper;
    }

    @Override
    protected QueryMapper getQueryMapper() {
        return queryMapper;
    }

    @Override
    protected void loadCache() {
        List<EditorImpl> editors = persistenceContext.getSortedList(TABLE, rowMapper);

        for (EditorImpl editor : editors) {
            getCache().put(editor.getId(), editor);
        }

        setCacheEntirelyLoaded(true);
    }

    @Override
    protected void load(int i) {
        EditorImpl editor = persistenceContext.getDataByID(TABLE, i, rowMapper);

        getCache().put(i, editor);
    }

    /**
     * A mapper to map resultset to editor.
     *
     * @author Baptiste Wicht
     */
    private static final class EditorRowMapper implements ParameterizedRowMapper<EditorImpl> {
        @Override
        public EditorImpl mapRow(ResultSet rs, int i) throws SQLException {
            EditorImpl editor = new EditorImpl();

            editor.setId(rs.getInt("ID"));
            editor.setName(rs.getString("NAME"));

            return editor;
        }
    }

    /**
     * A mapper to map resultset to editor.
     *
     * @author Baptiste Wicht
     */
    private static final class EditorQueryMapper implements QueryMapper {
        @Override
        public Query constructInsertQuery(Entity entity) {
            Editor editor = (Editor) entity;

            String query = "INSERT INTO " + TABLE + " (NAME) VALUES(?)";

            Object[] parameters = {
                    editor.getName()
            };

            return new Query(query, parameters);
        }

        @Override
        public Query constructUpdateQuery(Entity entity) {
            Editor editor = (Editor) entity;

            String query = "UPDATE " + TABLE + " SET NAME = ? WHERE ID = ?";

            Object[] parameters = {
                    editor.getName(),
                    editor.getId()};

            return new Query(query, parameters);
        }
    }
}