package org.jtheque.books.view.panels;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.books.persistence.od.AuthorImpl;
import org.jtheque.books.services.able.IAuthorsService;
import org.jtheque.books.view.able.IAuthorView;
import org.jtheque.books.view.controllers.able.IAuthorController;
import org.jtheque.books.view.fb.AuthorFormBean;
import org.jtheque.books.view.models.AuthorsModel;
import org.jtheque.books.view.toolbar.JPanelAuthorToolBar;
import org.jtheque.core.managers.error.JThequeError;
import org.jtheque.core.utils.ui.ValidationUtils;
import org.jtheque.core.utils.ui.PanelBuilder;
import org.jtheque.primary.od.CountryImpl;
import org.jtheque.primary.services.able.ICountriesService;
import org.jtheque.primary.view.able.ToolbarView;
import org.jtheque.primary.view.impl.components.panels.JThequeTitledPanel;
import org.jtheque.primary.view.impl.components.panels.PrincipalDataPanel;
import org.jtheque.utils.ui.GridBagUtils;
import org.jtheque.utils.ui.SwingUtils;
import org.jtheque.primary.view.impl.listeners.ObjectChangedEvent;
import org.jtheque.primary.view.impl.models.DataContainerCachedComboBoxModel;
import org.jtheque.primary.view.impl.models.NotesComboBoxModel;
import org.jtheque.primary.view.impl.renderers.JThequeTreeCellRenderer;
import org.jtheque.primary.view.impl.sort.SortManager;

import org.jdesktop.swingx.JXTree;
import org.jdesktop.swingx.JXTitledPanel;
import org.jdesktop.swingx.JXHyperlink;
import org.jdesktop.swingx.border.DropShadowBorder;

import javax.annotation.Resource;
import javax.annotation.PostConstruct;
import javax.swing.*;

import java.awt.Font;
import java.util.List;

/**
 * The principal view for authors.
 *
 * @author Baptiste Wicht
 */
public final class AuthorView extends PrincipalDataPanel implements IAuthorView {
    private static final long serialVersionUID = -7634054000892878297L;

    private JXTitledPanel authorsPanel;

    private JTextField fieldFirstname;
    private JTextField fieldName;
    private DataContainerCachedComboBoxModel<CountryImpl> countriesModel;
    private NotesComboBoxModel notesModel;

    private JXTree treeAuthors;

    private final SortManager sorter = new SortManager();

    private Action newCountryAction;
    private Action sortByCountry;
    private Action sortByNote;
    private JPanelAuthorToolBar toolBar;

    private JComboBox comboCountries;
    private JComboBox comboNote;

    @Resource
    private ICountriesService countriesService;

    @Resource
    private IAuthorController authorController;
    private JButton buttonNewCountry;

    @Override
    @PostConstruct
    public void build() {
        PanelBuilder builder = new PanelBuilder(this);
        
        buildListPanel(builder);
        buildSortPanel(builder);
        buildAuthorsPanel(builder);
        
        addListeners();
    }

    private void addListeners() {
        treeAuthors.addTreeSelectionListener(authorController);

        getModel().addDisplayListListener(this);
        getModel().addCurrentObjectListener(this);
    }

    /**
     * Build the internal panel for the list of authors.
     * 
     * @param parent The parent builder. 
     */
    private void buildListPanel(PanelBuilder parent) {
        JXTitledPanel panelListe = new JThequeTitledPanel("author.panel.list.title");
        panelListe.setBorder(new DropShadowBorder());
        panelListe.setTitleFont(panelListe.getTitleFont().deriveFont(Font.BOLD));

        PanelBuilder builder = new PanelBuilder();
        
        setTreeModel(sorter.createInitialModel(IAuthorsService.DATA_TYPE));

        treeAuthors = new JXTree(getTreeModel());
        treeAuthors.setCellRenderer(new JThequeTreeCellRenderer());
        treeAuthors.putClientProperty("JTree.lineStyle", "None");
        
        builder.addScrolled(treeAuthors, builder.gbcSet(0, 0, GridBagUtils.BOTH, GridBagUtils.FIRST_LINE_START, 1.0, 1.0));

        panelListe.setContentContainer(builder.getPanel());
        
        parent.add(panelListe, parent.gbcSet(0, 0, GridBagUtils.BOTH, GridBagUtils.FIRST_LINE_START, 0.25, 1.0));
    }

    /**
     * Build the internal panel for the sorts.
     * 
     * @param parent The parent builder. 
     */
    private void buildSortPanel(PanelBuilder parent) {
        JXTitledPanel panelTri = new JThequeTitledPanel("author.panel.sort.title");
        panelTri.setBorder(new DropShadowBorder());
        panelTri.setTitleFont(panelTri.getTitleFont().deriveFont(Font.BOLD));

        PanelBuilder builder = new PanelBuilder();
        
        JXHyperlink linkSortCountry = builder.add(new JXHyperlink(sortByCountry), 
                builder.gbcSet(0, 0, GridBagUtils.HORIZONTAL, GridBagUtils.BASELINE_LEADING, 1.0, 0.0));
        linkSortCountry.setClickedColor(linkSortCountry.getUnclickedColor());
        
        JXHyperlink linkSortNote = builder.add(new JXHyperlink(sortByNote), 
                builder.gbcSet(0, 1, GridBagUtils.HORIZONTAL, GridBagUtils.BASELINE_LEADING, 1.0, 0.0));
        linkSortNote.setClickedColor(linkSortNote.getUnclickedColor());

        panelTri.setContentContainer(builder.getPanel());
        
        parent.add(panelTri, parent.gbcSet(0, 1, GridBagUtils.HORIZONTAL, GridBagUtils.FIRST_LINE_START, 0.25, 0.0));
    }

    /**
     * Build the internal panel for the author.
     * 
     * @param parent the parent builder
     */
    private void buildAuthorsPanel(PanelBuilder parent) {
        authorsPanel = new JThequeTitledPanel("author.panel.author.title");
        authorsPanel.setBorder(new DropShadowBorder());
        authorsPanel.setTitleFont(authorsPanel.getTitleFont().deriveFont(Font.BOLD));

        PanelBuilder builder = new PanelBuilder();
        
        builder.add(toolBar, builder.gbcSet(0, 0, GridBagUtils.HORIZONTAL, GridBagUtils.BASELINE_LEADING, GridBagUtils.REMAINDER, 1, 1.0, 0.0));
        
        builder.addI18nLabel("author.firstname", builder.gbcSet(0, 1));
        
        fieldFirstname = builder.add(new JTextField(10), 
                builder.gbcSet(1, 1, GridBagUtils.NONE, GridBagUtils.BASELINE_LEADING, GridBagUtils.REMAINDER, 1, 1.0, 0.0));
        SwingUtils.addFieldLenghtLimit(fieldFirstname, 80);
        fieldFirstname.setEnabled(false);
        
        builder.addI18nLabel("author.name", builder.gbcSet(0, 2));
        
        fieldName = builder.add(new JTextField(10), 
                builder.gbcSet(1, 2, GridBagUtils.NONE, GridBagUtils.BASELINE_LEADING, GridBagUtils.REMAINDER, 1, 1.0, 0.0));
        SwingUtils.addFieldLenghtLimit(fieldName, 80);
        fieldName.setEnabled(false);

        builder.addI18nLabel("author.country", builder.gbcSet(0, 3));

        countriesModel = new DataContainerCachedComboBoxModel<CountryImpl>(countriesService);

        comboCountries = builder.addComboBox(countriesModel, builder.gbcSet(1, 3));
        comboCountries.setEnabled(false);

        buttonNewCountry = builder.addButton(newCountryAction, 
                builder.gbcSet(2, 3, GridBagUtils.NONE, GridBagUtils.BASELINE_LEADING, GridBagUtils.REMAINDER, 1, 1.0, 0.0));
        buttonNewCountry.setEnabled(false);
        
        builder.addI18nLabel("author.note", builder.gbcSet(0, 4));

        notesModel = new NotesComboBoxModel();

        comboNote = builder.addComboBox(notesModel, 
                builder.gbcSet(1, 4, GridBagUtils.NONE, GridBagUtils.BASELINE_LEADING, GridBagUtils.REMAINDER, GridBagUtils.REMAINDER, 1.0, 1.0));
        comboNote.setEnabled(false);

        authorsPanel.setContentContainer(builder.getPanel());
        
        parent.add(authorsPanel, parent.gbcSet(1, 0, GridBagUtils.BOTH, GridBagUtils.FIRST_LINE_START, 1, 2, 0.75, 1.0));
    }

    @Override
    public String getDataType() {
        return IAuthorsService.DATA_TYPE;
    }

    @Override
    protected JXTree getTree() {
        return treeAuthors;
    }

    @Override
    public JComponent getImpl() {
        return this;
    }

    @Override
    public Integer getPosition() {
        return 1;
    }

    @Override
    public String getTitlekey() {
        return "data.titles.author";
    }

    @Override
    public ToolbarView getToolbarView() {
        return toolBar;
    }

    @Override
    public AuthorsModel getModel() {
        return (AuthorsModel) super.getModel();
    }

    @Override
    public boolean isEnabled() {
        return getModel().isEnabled();
    }

    @Override
    public void setEnabled(boolean enabled) {
        fieldFirstname.setEnabled(enabled);
        fieldName.setEnabled(enabled);
        buttonNewCountry.setEnabled(enabled);
        comboNote.setEnabled(enabled);
        comboCountries.setEnabled(enabled);
    }

    @Override
    public void objectChanged(ObjectChangedEvent event) {
        setCurrentAuthor((AuthorImpl) event.getObject());
    }

    /**
     * Set the current author.
     *
     * @param author The current author.
     */
    private void setCurrentAuthor(AuthorImpl author) {
        authorsPanel.setTitle(author.getAffichableText());

        fieldFirstname.setText(author.getFirstName());
        fieldName.setText(author.getName());
        countriesModel.setSelectedItem(author.getTheCountry());
        notesModel.setSelectedItem(author.getNote());
    }

    @Override
    public void clear() {
        authorsPanel.setTitle(getMessage("author.panel.author.title"));

        fieldFirstname.setText("");
        fieldName.setText("");
        countriesModel.setSelectedItem(null);
        notesModel.setSelectedItem(null);
    }

    /**
     * Fill a <code>AuthorFormBean</code> with the infos in the interface.
     *
     * @return The filled <code>AuthorFormBean</code>.
     */
    @Override
    public AuthorFormBean fillFilmFormBean() {
        AuthorFormBean fb = new AuthorFormBean();

        fb.setName(fieldName.getText());
        fb.setFirstName(fieldFirstname.getText());
        fb.setNote(notesModel.getSelectedNote());
        fb.setCountry(countriesModel.getSelectedData());

        return fb;
    }

    @Override
    protected void validate(List<JThequeError> errors) {
        ValidationUtils.rejectIfEmpty(fieldName.getText(), "author.name", errors);
        ValidationUtils.rejectIfEmpty(fieldFirstname.getText(), "author.firstname", errors);

        ValidationUtils.rejectIfLongerThan(fieldName.getText(), "author.name", 100, errors);
        ValidationUtils.rejectIfLongerThan(fieldFirstname.getText(), "author.firstname", 100, errors);

        ValidationUtils.rejectIfNothingSelected(notesModel, "author.note", errors);
        ValidationUtils.rejectIfNothingSelected(countriesModel, "author.country", errors);
    }

    @Override
    public JComponent getComponent() {
        return this;
    }

    public void setToolBar(JPanelAuthorToolBar toolBar) {
        this.toolBar = toolBar;
    }

    public void setSortByNote(Action sortByNote) {
        this.sortByNote = sortByNote;
    }

    public void setSortByCountry(Action sortByCountry) {
        this.sortByCountry = sortByCountry;
    }

    public void setNewCountryAction(Action newCountryAction) {
        this.newCountryAction = newCountryAction;
    }
}