package org.jtheque.books.view.panels;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.books.persistence.od.AuthorImpl;
import org.jtheque.books.persistence.od.EditorImpl;
import org.jtheque.primary.od.SagaImpl;
import org.jtheque.books.persistence.od.abstraction.Book;
import org.jtheque.books.services.able.IEditorsService;
import org.jtheque.primary.services.able.ISagasService;
import org.jtheque.books.view.fb.BookFormBean;
import org.jtheque.books.view.models.list.AuthorsListModel;
import org.jtheque.books.view.models.list.SimpleAuthorsModel;
import org.jtheque.core.managers.error.JThequeError;
import org.jtheque.core.managers.view.impl.actions.JThequeSimpleAction;
import org.jtheque.core.managers.view.impl.actions.JThequeAction;
import org.jtheque.core.utils.ui.ValidationUtils;
import org.jtheque.core.utils.ui.PanelBuilder;
import org.jtheque.primary.od.LanguageImpl;
import org.jtheque.primary.services.able.ILanguagesService;
import org.jtheque.utils.ui.SwingUtils;
import org.jtheque.utils.ui.GridBagUtils;
import org.jtheque.primary.view.impl.listeners.ObjectChangedEvent;
import org.jtheque.primary.view.impl.models.DataContainerCachedComboBoxModel;

import org.jdesktop.swingx.JXPanel;

import javax.annotation.Resource;
import javax.annotation.PostConstruct;
import javax.swing.*;
import javax.swing.text.DefaultFormatter;
import java.text.NumberFormat;
import java.util.Collection;
import java.util.List;

/**
 * The panel "infos" of the books view.
 *
 * @author Baptiste Wicht
 */
public final class JPanelBookInfos extends JXPanel implements IInfosPanel {
    private static final long serialVersionUID = 6833310794095034469L;

    private DataContainerCachedComboBoxModel<EditorImpl> editorsModel;
    private DataContainerCachedComboBoxModel<LanguageImpl> languagesModel;
    private DataContainerCachedComboBoxModel<SagaImpl> sagasModel;

    private JComboBox comboEditors;
    private JComboBox comboLanguages;
    private JComboBox comboSagas;

    private JFormattedTextField fieldYear;
    private JFormattedTextField fieldPages;

    private JThequeAction newEditorAction;
    private JThequeAction newLanguageAction;
    private JThequeAction newSagaAction;
    
    private JThequeSimpleAction addAuthorAction;
    private JThequeSimpleAction removeAuthorAction;
    
    private JList authorsList;
    private JList authorsBookList;

    private AuthorsListModel authorsModel;
    private SimpleAuthorsModel authorsBookModel;
    
    private JButton buttonNewEditor;
    private JButton buttonNewLanguage;
    private JButton buttonNewSaga;

    @Resource
    private ILanguagesService languagesService;

    @Resource
    private IEditorsService editorsService;

    @Resource
    private ISagasService sagasService;

    /**
     * Build the view.
     */
    @PostConstruct
    private void build() {
        PanelBuilder builder = new PanelBuilder(this);
        
        builder.addI18nLabel("book.editor", builder.gbcSet(0, 0));

        editorsModel = new DataContainerCachedComboBoxModel<EditorImpl>(editorsService);
        
        comboEditors = builder.addComboBox(editorsModel, builder.gbcSet(1, 0));
        comboEditors.setEnabled(false);

        buttonNewEditor = builder.addButton(newEditorAction, builder.gbcSet(2, 0, GridBagUtils.NONE, GridBagUtils.REMAINDER, 1));
        buttonNewEditor.setEnabled(false);
        
        builder.addI18nLabel("book.language", builder.gbcSet(0, 1));

        languagesModel = new DataContainerCachedComboBoxModel<LanguageImpl>(languagesService);

        comboLanguages = builder.addComboBox(languagesModel, builder.gbcSet(1, 1));
        comboLanguages.setEnabled(false);

        buttonNewLanguage = builder.addButton(newLanguageAction, builder.gbcSet(2, 1, GridBagUtils.NONE, GridBagUtils.REMAINDER, 1));
        buttonNewLanguage.setEnabled(false);
        
        builder.addI18nLabel("book.saga", builder.gbcSet(0, 2));

        sagasModel = new DataContainerCachedComboBoxModel<SagaImpl>(sagasService);

        comboSagas = builder.addComboBox(sagasModel, builder.gbcSet(1, 2));
        comboSagas.setEnabled(false);

        buttonNewSaga = builder.addButton(newSagaAction, builder.gbcSet(2, 2, GridBagUtils.NONE, GridBagUtils.REMAINDER, 1));
        buttonNewSaga.setEnabled(false);
        
        builder.addI18nLabel("book.year", builder.gbcSet(0, 3));
        
        NumberFormat format = NumberFormat.getNumberInstance();
        format.setParseIntegerOnly(true);
        format.setGroupingUsed(false);

        fieldYear = builder.add(new JFormattedTextField(format), builder.gbcSet(1, 3));
        fieldYear.setColumns(5);
        ((DefaultFormatter) fieldYear.getFormatter()).setAllowsInvalid(false);
        SwingUtils.addFieldLenghtLimit(fieldYear, 4);
        fieldYear.setEnabled(false);
        
        builder.addI18nLabel("book.pages", builder.gbcSet(0, 4));

        fieldPages = builder.add(new JFormattedTextField(format), builder.gbcSet(1, 4));
        fieldPages.setColumns(5);
        fieldPages.setEnabled(false);

        addAuthorsPanel(builder);
    }

    private void addAuthorsPanel(PanelBuilder parent) {
        PanelBuilder builder = new PanelBuilder();
        
        authorsModel = new AuthorsListModel();

        authorsList = new JList(authorsModel);
        authorsList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

        builder.addScrolled(authorsList, builder.gbcSet(0, 0, GridBagUtils.BOTH, GridBagUtils.BASELINE_LEADING, 1, GridBagUtils.REMAINDER, 0.5, 1.0));
        
        addAuthorAction.setEnabled(false);
        builder.addButton(addAuthorAction, builder.gbcSet(1, 0));
        
        removeAuthorAction.setEnabled(false);
        builder.addButton(removeAuthorAction, builder.gbcSet(1, 1));
        
        authorsBookModel = new SimpleAuthorsModel();

        authorsBookList = new JList(authorsBookModel);
        authorsBookList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

        builder.addScrolled(authorsBookList, builder.gbcSet(2, 0, GridBagUtils.BOTH, GridBagUtils.BASELINE_LEADING, 1, GridBagUtils.REMAINDER, 0.5, 1.0));

        parent.add(builder.getPanel(), parent.gbcSet(0, 5, GridBagUtils.BOTH, GridBagUtils.BASELINE_LEADING, 3, 1, 1.0, 1.0));
    }

    @Override
    public void objectChanged(ObjectChangedEvent event) {
        Book book = (Book)event.getObject();

        comboEditors.setSelectedItem(book.getTheEditor());
        comboLanguages.setSelectedItem(book.getTheLanguage());
        comboSagas.setSelectedItem(book.getTheSaga());

        fieldYear.setText(Integer.toString(book.getYear()));
        fieldPages.setText(Integer.toString(book.getPages()));

        authorsBookModel.removeAllElements();
        authorsModel.reload();

        List<AuthorImpl> authors = book.getAuthors();

        for (AuthorImpl author : authors) {
            authorsModel.removeElement(author);
            authorsBookModel.addElement(author);
        }
    }

    @Override
    public void setEnabled(boolean enabled) {
        comboEditors.setEnabled(enabled);
        comboLanguages.setEnabled(enabled);
        comboSagas.setEnabled(enabled);

        fieldYear.setEnabled(enabled);
        fieldPages.setEnabled(enabled);

        buttonNewLanguage.setEnabled(enabled);
        buttonNewEditor.setEnabled(enabled);
        buttonNewSaga.setEnabled(enabled);
        addAuthorAction.setEnabled(enabled);
        removeAuthorAction.setEnabled(enabled);

        authorsList.setEnabled(enabled);
        authorsBookList.setEnabled(enabled);
    }

    @Override
    public DefaultListModel getAuthorsModel() {
        return authorsModel;
    }

    @Override
    public DefaultListModel getAuthorsBookModel() {
        return authorsBookModel;
    }

    @Override
    public int[] getSelectedAuthorsIndices() {
        return authorsList.getSelectedIndices();
    }

    @Override
    public int[] getSelectedAuthorsBookIndices() {
        return authorsBookList.getSelectedIndices();
    }

    @Override
    public void fillFormBean(BookFormBean formBean) {
        formBean.setTheEditor(editorsModel.getSelectedData());
        formBean.setTheLanguage(languagesModel.getSelectedData());
        formBean.setTheSaga(sagasModel.getSelectedData());
        formBean.setYear(Integer.parseInt(fieldYear.getText()));
        formBean.setPages(Integer.parseInt(fieldPages.getText()));
        formBean.setAuthors(authorsBookModel.getAuthors());
    }

    @Override
    public void clear() {
        comboEditors.setSelectedItem(null);
        comboLanguages.setSelectedItem(null);
        comboSagas.setSelectedItem(null);

        fieldYear.setText("0");
        fieldPages.setText("0");

        authorsBookModel.removeAllElements();
        authorsModel.reload();
    }

    @Override
    public void validate(Collection<JThequeError> errors) {
        ValidationUtils.rejectIfNothingSelected(languagesModel, "book.language", errors);

        ValidationUtils.rejectIfNotNumerical(fieldYear.getText(), "book.year", errors);
        ValidationUtils.rejectIfNotNumerical(fieldPages.getText(), "book.pages", errors);
    }
    
    @Override
    public JComponent getImpl(){
        return this;
    }

    public void setRemoveAuthorAction(JThequeSimpleAction removeAuthorAction) {
        this.removeAuthorAction = removeAuthorAction;
    }

    public void setAddAuthorAction(JThequeSimpleAction addAuthorAction) {
        this.addAuthorAction = addAuthorAction;
    }

    public void setNewLanguageAction(JThequeAction newLanguageAction) {
        this.newLanguageAction = newLanguageAction;
    }

    public void setNewEditorAction(JThequeAction newEditorAction) {
        this.newEditorAction = newEditorAction;
    }

    public void setNewSagaAction(JThequeAction newSagaAction) {
        this.newSagaAction = newSagaAction;
    }
}